package com.example.samplemqttrequestorapp;

import android.os.AsyncTask;

import com.ibm.micro.client.mqttv3.MqttCallback;
import com.ibm.micro.client.mqttv3.MqttClient;
import com.ibm.micro.client.mqttv3.MqttDefaultFilePersistence;
import com.ibm.micro.client.mqttv3.MqttDeliveryToken;
import com.ibm.micro.client.mqttv3.MqttException;
import com.ibm.micro.client.mqttv3.MqttMessage;
import com.ibm.micro.client.mqttv3.MqttTopic;

public class Requestor extends AsyncTask<Void, Void, Void> implements MqttCallback {

	private String topicName = "T1";
	private int qos = 2;
	private String url = "tcp://10.0.2.2:1883";
	private String clientId;
	private String message = "BakedBeans";
	private MqttClient client;
	private MainActivity activity;

	public Requestor(MainActivity activity, String clientId) {
		this.activity = activity;
		this.clientId = clientId;
	}

	@Override
	protected synchronized Void doInBackground(Void... params) {
		try {
			String tmpDir = System.getProperty("java.io.tmpdir");
			MqttDefaultFilePersistence dataStore = new MqttDefaultFilePersistence(tmpDir);

			// Construct the MqttClient instance
			client = new MqttClient(url, clientId, dataStore);
			// Set this wrapper as the callback handler
			client.setCallback(this);

			connect();

		} catch (MqttException e) {
			e.printStackTrace();
			System.out.println("Unable to set up client: " + e.toString());
		}
		return null;
	}

	private void connect() {
		// Keep trying to connect
		while (true) {
			try {
				client.connect();
				client.subscribe(clientId, qos);
				break;
			} catch (MqttException e) {
				e.printStackTrace();
				try {
					Thread.sleep(5000);
				} catch (InterruptedException ie) {
					ie.printStackTrace();
				}
			}
		}

		// Tick the "Connected" checkbox
		activity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Requestor.this.activity.setConnected(true);
			}
		});
	}
	
	public void disconnect() {
		try {
			client.disconnect();
		} catch (MqttException e) {
			e.printStackTrace();
		}
	}

	public synchronized void publish() throws MqttException {
		if(client == null)
			return;
		MqttTopic topic = client.getTopic(topicName);
		MqttMessage m = new MqttMessage((message + "," + clientId).getBytes());
		m.setQos(qos);
		topic.publish(m);
	}

	@Override
	public void connectionLost(Throwable arg0) {
		activity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				activity.setConnected(false);
			}
		});

		connect();
	}

	@Override
	public void deliveryComplete(MqttDeliveryToken arg0) {
	}

	@Override
	public void messageArrived(MqttTopic arg0, final MqttMessage message) throws Exception {
		// Change the text displayed to match the message
		activity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				activity.setText(new String(message.getPayload()));
			}
		});
	}

}
